/* Include STM32 Firmware Lib Headers */
#include "stm32f10x.h"


/* Scheduler include files. */
#include "FreeRTOS.h"
#include "task.h"
#include "croutine.h"


// Function Declarations
int main(void);
void Led_Init(void);
void Usart1_Init(void);
void uart_putc(char c);
void uart_puts(char *c);
void vLedsFloat(void* dummy);
void vLedsFlash(void* dummy);
void vPrintUart(void* dummy);


/* Main function */
int main(void){

    /* Init Hardware */
    Led_Init();
 	Usart1_Init();

    /* Create Demo Tasks */
    if (!(pdPASS == xTaskCreate( vLedsFloat, ( signed char * ) "LedFloat", 100, NULL, 10, NULL ))) goto hell;
    if (!(pdPASS == xTaskCreate( vLedsFlash, ( signed char * ) "LedFlash", 100, NULL, 10, NULL ))) goto hell;
	if (!(pdPASS == xTaskCreate( vPrintUart, ( signed char * ) "Uart",     100, NULL, 10, NULL ))) goto hell;

	vTaskStartScheduler();
	/* should never be reached */
hell:
	/* Here the devil lives */
	while(1);
    return 0;
}


/* Controls 4 LEDs (chaser light) */
void vLedsFloat(void* dummy){
    
	uint32_t c=1;
	uint32_t b=1;
    while(1){
        b<<=1;
        while(42){
            if (c<(1<<3)) c<<=1;
            else break;
            GPIOB->BRR  |= 0x00000f00;
    		GPIOB->BSRR |= (0x00000000|(c<<8))&0x00000f00;
            vTaskDelay(50/portTICK_RATE_MS);
        }
        while(42){
          if (c>1) c>>=1;
          else break;
          GPIOB->BRR  |= 0x00000f00;
    	  GPIOB->BSRR |= (0x00000000|(c<<8))&0x00000f00;
          vTaskDelay(50/portTICK_RATE_MS);
        }
    }
}


/* Controls the 4 other LEDs and let them blink */
void vLedsFlash(void* dummy){
    
	while(1){
        GPIOB->BRR  |= 0x0000f000;
        vTaskDelay(57/portTICK_RATE_MS);
        GPIOB->BSRR |= 0x0000f000;
        vTaskDelay(57/portTICK_RATE_MS);

        }
}


/* Writes "Hallo\r\n" each 500 ms */
void vPrintUart(void* dummy)
{
	portTickType last_wakeup_time;
	last_wakeup_time = xTaskGetTickCount();
	while(1){
		uart_puts("Hallo\r\n");
		vTaskDelayUntil(&last_wakeup_time, 500/portTICK_RATE_MS);
	}
}


/* Inits USART1 (serial line) */
void Usart1_Init(void){

    /* Firmware library specific Init-Structs */
	USART_InitTypeDef       usart_i;
    USART_ClockInitTypeDef  usart_c;
	GPIO_InitTypeDef 		gpio_i;

	GPIO_PinRemapConfig(GPIO_Remap_USART1,DISABLE);

	/* Enable needed clocks */
	RCC_APB2PeriphClockCmd( RCC_APB2Periph_AFIO,   ENABLE);
	RCC_APB2PeriphClockCmd( RCC_APB2Periph_USART1, ENABLE);
	RCC_APB2PeriphClockCmd( RCC_APB2Periph_GPIOA,  ENABLE);

	/* TX - GPIO*/
	gpio_i.GPIO_Pin   = GPIO_Pin_9;
	gpio_i.GPIO_Speed = GPIO_Speed_50MHz;
	gpio_i.GPIO_Mode  = GPIO_Mode_AF_PP;
	GPIO_Init( GPIOA, &gpio_i);

	/* RX - GPIO*/
	gpio_i.GPIO_Pin   = GPIO_Pin_10;
	gpio_i.GPIO_Speed = GPIO_Speed_50MHz;
	gpio_i.GPIO_Mode  = GPIO_Mode_IN_FLOATING;
	GPIO_Init( GPIOA, &gpio_i);

	/* Configure UART 115200-8N1 */
	usart_i.USART_BaudRate              = 115200;
    usart_i.USART_HardwareFlowControl   = USART_HardwareFlowControl_None;
    usart_i.USART_Mode					= USART_Mode_Rx | USART_Mode_Tx;
    usart_i.USART_Parity				= USART_Parity_No;
    usart_i.USART_StopBits				= USART_StopBits_1;
    usart_i.USART_WordLength            = USART_WordLength_8b;

    usart_c.USART_Clock					= USART_Clock_Enable;
    usart_c.USART_CPHA					= USART_CPHA_1Edge;
    usart_c.USART_CPOL					= USART_CPOL_Low;
    usart_c.USART_LastBit				= USART_LastBit_Disable;

	/* Write configuration to registers */
	USART_ClockInit(USART1, &usart_c);
	USART_Init(USART1, &usart_i);
	USART_Cmd(USART1,ENABLE);

}


/* Configure the LED GPIOs*/
void Led_Init(void){

    GPIO_InitTypeDef GPIO_InitStructure;
    
	/* init Clocks */
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);

    /* Configure PB8-PB15 as output push-pull (LED) */
    GPIO_WriteBit(GPIOB,0x0000ff00,Bit_SET);
    GPIO_InitStructure.GPIO_Pin =  0x0000ff00;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitStructure);

}


/* Writes one character over the serial line*/
void uart_putc(char c){

	while(USART_GetFlagStatus(USART1, USART_FLAG_TXE) == RESET);
	USART_SendData(USART1, c);
}


/* Writes a zero teminated string over the serial line*/
void uart_puts(char *c){

	while(*c!=0)
		uart_putc(*(c++));
}

